﻿using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using Clio.Utilities;
using RetBot.Settings;

namespace RetBot
{
    public static class MovementActions
    {
        private const int MAX_TICKS = 100;
        private const int WAIT_TIME = 200;
        private const int MOVE_TIMEOUT_MS = 900000;
        public static RetBotSettings settings = RetBotSettings.instance;

        #region Movement Action

        public static async Task<bool> MoveTo(Vector3 location)
        {

            var stopwatch = new Stopwatch();
            stopwatch.Start();

            while (stopwatch.ElapsedMilliseconds < MOVE_TIMEOUT_MS && Core.Me.IsAlive)
            {
                if (Navigator.NavigationProvider != null)
                {
                    Navigator.MoveTo(location);
                }
                else
                {
                    break;
                }

                float distance = Core.Me.Location.Distance(location);
                if (distance < 0.8f)
                {
                    stopwatch.Stop();
                    Navigator.Stop();
                    return true;
                }

                if (Core.Me.IsMounted && distance < 20f)
                {
                    await Dismount();
                }
                else
                {
                    await Coroutine.Yield();
                }
            }

            Navigator.Stop();
            stopwatch.Stop();
            return false;
        }

        public static async Task<bool> Mount()
        {
            int ticks = 0;
            while (ticks < MAX_TICKS && !Core.Me.IsMounted && !Core.Me.InCombat && !WorldManager.InSanctuary)
            {
                Actionmanager.Mount();
                await Coroutine.Sleep(WAIT_TIME);
                ticks++;
            }

            return Core.Me.IsMounted;
        }

        public static async Task<bool> Dismount()
        {
            int ticks = 0;
            while (ticks < MAX_TICKS && Core.Me.IsAlive && Core.Me.IsMounted)
            {
                Actionmanager.Dismount();
                await Coroutine.Sleep(WAIT_TIME);
                ticks++;
            }

            return !Core.Me.IsMounted;
        }

        #endregion


        public static async Task RetainerMove()
        {
            var initialLocation = Core.Me.Location;
            uint initialmap = WorldManager.ZoneId;
            uint zoneid = CurrentMap(WorldManager.ZoneId);

            if (WorldManager.ZoneId != 156)
            {
                Logging.Write("[Retbot] Going to the Summoning Bell now!");
                if (Core.Player.IsMounted)
                {
                    Actionmanager.Dismount();
                    Thread.Sleep(3000);
                }
                WorldManager.TeleportById(24);
                Thread.Sleep(10000);
            }

            var location = new Vector3(36.32549f, 28.99997f, -762.0803f);
            await MoveTo(location);
            await RetainerRetask.Retask();
            await Coroutine.Sleep(2000);

            Logging.Write("[Retbot] Returning to Initial Location.");

            if (WorldManager.ZoneId == 156)
            {
                Thread.Sleep(2000);
            }
            else if (CurrentMap(WorldManager.ZoneId) != initialmap)
            {
                WorldManager.TeleportById(zoneid);
                Thread.Sleep(10000);
            }

            MovementManager.Jump();
            await MoveTo(initialLocation);

            Logging.Write("[Retbot] Sleeping for 62 minutes.");
            await Coroutine.Sleep(3720000);
        }

        public static uint CurrentMap(uint zoneid)
        {
            switch (zoneid)
            {
                case 148:
                    return 3; // "Bentbranch Meadows";//148;

                case 154:
                    return 7; // "Fallgourd Float"; //154;

                case 152:
                    return 4; // "The Hawthorne Hut";//152;

                case 146:
                    return 20; // "Forgotten Springs";//146;

                case 145:
                    return 18; // "Camp Drybone";//145;

                case 140:
                    return 17; // "Horizon";//140;

                case 141:
                    return 53; // "Black Brush Station";//141;

                case 139:
                    return 15; // "Camp Bronze Lake";//139;

                case 134:
                    return 52; // "Summerford Farms";//134;

                case 180:
                    return 16; // "Camp Overlook";//180;

                case 135:
                    return 10; // "Moraby Drydocks";//135;

                case 138:
                    return 14; // "Aleport";//138;

                case 132:
                    return 2; // New Gridania

                case 153:
                    return 5; // Quarrymill/Camp Tranquill

                case 129:
                    return 8; // Limsa Lominsa Lower Decks

                case 130:
                    return 9; // Steps of Nald

                case 137:
                    return 11; // Costa Del Sol

                case 147:
                    return 22; // Ceruleum Processing Plant

                case 155:
                    return 23; // Camp Dragonhead

                case 156:
                    return 24; // Revenant's Toll

                case 250:
                    return 55; // Wolves Den

                case 340:
                    return 56; // Gridania Housing

                case 144:
                    return 62; // Gold Saucer

                default:
                    return 0;
            }
        }
    }
}
    

